// Generated by CoffeeScript 1.7.1
var ATEM, EventEmitter, dgram,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

dgram = require('dgram');

EventEmitter = (require('events')).EventEmitter;

ATEM = (function() {
  var AUDIO_GAIN_RATE, COMMAND_CONNECT_HELLO, COMMAND_CONNECT_HELLO_ANSWER, DEBUG, DEFAULT_PORT, RECONNECT_INTERVAL;

  DEBUG = process.env['ATEM_DEBUG'] ? process.env['ATEM_DEBUG'] === 'true' : false;

  DEFAULT_PORT = 9910;

  RECONNECT_INTERVAL = 5000;

  COMMAND_CONNECT_HELLO = [0x10, 0x14, 0x53, 0xAB, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3A, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];

  COMMAND_CONNECT_HELLO_ANSWER = [0x80, 0x0C, 0x53, 0xAB, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00];

  AUDIO_GAIN_RATE = 65381;

  ATEM.Model = {
    'TVS': 0x01,
    '1ME': 0x02,
    '2ME': 0x03,
    'PS4K': 0x04,
    '1ME4K': 0x05,
    '2ME4K': 0x06,
    '2MEBS4K': 0x07
  };

  ATEM.TransitionStyle = {
    MIX: 0x00,
    DIP: 0x01,
    WIPE: 0x02,
    DVE: 0x03,
    STING: 0x04
  };

  ATEM.TallyState = {
    None: 0x00,
    Program: 0x01,
    Preview: 0x02
  };

  ATEM.PacketFlag = {
    Sync: 0x01,
    Connect: 0x02,
    Repeat: 0x04,
    Unknown: 0x08,
    Ack: 0x10
  };

  ATEM.prototype.state = {
    tallys: [],
    channels: {},
    video: {
      upstreamKeyNextState: [],
      upstreamKeyState: [],
      downstreamKeyOn: [],
      downstreamKeyTie: [],
      auxs: {}
    },
    audio: {
      channels: {}
    }
  };

  ATEM.prototype.connected = false;

  ATEM.prototype.localPackedId = 1;

  ATEM.prototype.sessionId = [];

  function ATEM() {
    this._receivePacket = __bind(this._receivePacket, this);
    this.event = new EventEmitter;
    this.event.on('ping', (function(_this) {
      return function(err) {
        return _this.lastConnectAt = new Date().getTime();
      };
    })(this));
    setInterval((function(_this) {
      return function() {
        if (_this.lastConnectAt + RECONNECT_INTERVAL > new Date().getTime()) {
          return;
        }
        if (_this.connected) {
          _this.connected = false;
          _this.event.emit('disconnect', null, null);
        }
        _this.localPackedId = 1;
        _this.sessionId = [];
        return _this.connect(_this.address, _this.port);
      };
    })(this), RECONNECT_INTERVAL);
  }

  ATEM.prototype.connect = function(address, port, local_port) {
    this.address = address;
    this.port = port != null ? port : DEFAULT_PORT;
    if (local_port == null) {
      local_port = 0;
    }
    local_port || (local_port = 1024 + Math.floor(Math.random() * 64511));
    this.socket = dgram.createSocket('udp4');
    this.socket.on('message', this._receivePacket);
    this.socket.bind(local_port);
    return this._sendPacket(COMMAND_CONNECT_HELLO);
  };

  ATEM.prototype.on = function(name, callback) {
    return this.event.on(name, callback);
  };

  ATEM.prototype.once = function(name, callback) {
    return this.event.once(name, callback);
  };

  ATEM.prototype._sendCommand = function(command, options) {
    var byte, i, message, _i, _len;
    message = [];
    message[0] = (20 + options.length) / 256 | 0x08;
    message[1] = (20 + options.length) % 256;
    message[2] = this.sessionId[0];
    message[3] = this.sessionId[1];
    message[10] = this.localPackedId / 256;
    message[11] = this.localPackedId % 256;
    message[12] = (8 + options.length) / 256;
    message[13] = (8 + options.length) % 256;
    message[16] = command.charCodeAt(0);
    message[17] = command.charCodeAt(1);
    message[18] = command.charCodeAt(2);
    message[19] = command.charCodeAt(3);
    for (i = _i = 0, _len = options.length; _i < _len; i = ++_i) {
      byte = options[i];
      message[20 + i] = byte;
    }
    this._sendPacket(message);
    return this.localPackedId++;
  };

  ATEM.prototype._sendPacket = function(message) {
    var buffer;
    buffer = new Buffer(message);
    if (DEBUG) {
      console.log('SEND', buffer);
    }
    return this.socket.send(buffer, 0, buffer.length, this.port, this.address);
  };

  ATEM.prototype._receivePacket = function(message, remote) {
    var flags, length;
    length = ((message[0] & 0x07) << 8) | message[1];
    if (length !== remote.size) {
      return;
    }
    flags = message[0] >> 3;
    this.sessionId = [message[2], message[3]];
    if (flags & ATEM.PacketFlag.Connect) {
      this._sendPacket(COMMAND_CONNECT_HELLO_ANSWER);
      if (!this.connected) {
        this.connected = true;
        this.event.emit('connect', null, null);
      }
    } else if (flags & ATEM.PacketFlag.Sync) {
      this._sendPacket([0x80, 0x0C, this.sessionId[0], this.sessionId[1], message[10], message[11], 0x00, 0x00, 0x00, 0x41, 0x00, 0x00]);
      this.event.emit('ping', null, null);
    }
    if (remote.size > 12 && remote.size !== 20) {
      this._parseCommand(message.slice(12));
      return this.event.emit('stateChanged', null, this.state);
    }
  };

  ATEM.prototype._parseCommand = function(buffer) {
    var length, name;
    length = this._parseNumber(buffer.slice(0, 2));
    name = this._parseString(buffer.slice(4, 8));
    if (DEBUG) {
      console.log('RECV', "" + name + "(" + length + ")", buffer.slice(0, length));
    }
    this._setStatus(name, buffer.slice(0, length).slice(8));
    if (buffer.length > length) {
      return this._parseCommand(buffer.slice(length));
    }
  };

  ATEM.prototype._setStatus = function(name, buffer) {
    var aux, channel, channelMappings, i, leftGain, numberOfChannels, offset, rightGain, _i, _j, _k, _results;
    switch (name) {
      case '_ver':
        this.state._ver0 = buffer[1];
        return this.state._ver1 = buffer[3];
      case '_pin':
        this.state._pin = this._parseString(buffer);
        return this.state.model = buffer[40];
      case 'InPr':
        channel = this._parseNumber(buffer.slice(0, 2));
        return this.state.channels[channel] = {
          name: this._parseString(buffer.slice(2, 22)),
          label: this._parseString(buffer.slice(22, 26))
        };
      case 'PrgI':
        return this.state.video.programInput = this._parseNumber(buffer.slice(2, 4));
      case 'PrvI':
        return this.state.video.previewInput = this._parseNumber(buffer.slice(2, 4));
      case 'TrPr':
        return this.state.video.transitionPreview = buffer[1] > 0 ? true : false;
      case 'TrPs':
        this.state.video.transitionPosition = this._parseNumber(buffer.slice(4, 6)) / 10000;
        return this.state.video.transitionFrameCount = buffer[2];
      case 'TrSS':
        this.state.video.transitionStyle = this._parseNumber(buffer.slice(0, 2));
        this.state.video.upstreamKeyNextBackground = (buffer[2] >> 0 & 1) === 0x01;
        this.state.video.upstreamKeyNextState[0] = (buffer[2] >> 1 & 1) === 0x01;
        if (this.state.model !== ATEM.Model.TVS && this.state.model !== ATEM.Model.PS4K) {
          this.state.video.upstreamKeyNextState[1] = (buffer[2] >> 2 & 1) === 0x01;
          this.state.video.upstreamKeyNextState[2] = (buffer[2] >> 3 & 1) === 0x01;
          return this.state.video.upstreamKeyNextState[3] = (buffer[2] >> 4 & 1) === 0x01;
        }
        break;
      case 'DskS':
        return this.state.video.downstreamKeyOn[buffer[0]] = buffer[1] === 1 ? true : false;
      case 'DskP':
        return this.state.video.downstreamKeyTie[buffer[0]] = buffer[1] === 1 ? true : false;
      case 'KeOn':
        return this.state.video.upstreamKeyState[buffer[1]] = buffer[2] === 1 ? true : false;
      case 'FtbS':
        return this.state.video.fadeToBlack = buffer[1] > 0 ? true : false;
      case 'TlIn':
        return this.state.tallys = this._bufferToArray(buffer.slice(2));
      case 'AuxS':
        aux = buffer[0];
        return this.state.video.auxs[aux] = this._parseNumber(buffer.slice(2, 4));
      case 'AMIP':
        channel = this._parseNumber(buffer.slice(0, 2));
        return this.state.audio.channels[channel] = {
          on: buffer[8] === 1 ? true : false,
          afv: buffer[8] === 2 ? true : false,
          gain: this._parseNumber(buffer.slice(10, 12)) / AUDIO_GAIN_RATE,
          rawGain: this._parseNumber(buffer.slice(10, 12)),
          rawPan: this._parseNumber(buffer.slice(12, 14))
        };
      case 'AMMO':
        return this.state.audio.master = {
          afv: buffer[4] === 1 ? true : false,
          gain: this._parseNumber(buffer.slice(0, 2)) / AUDIO_GAIN_RATE,
          rawGain: this._parseNumber(buffer.slice(0, 2))
        };
      case 'AMLv':
        numberOfChannels = this._parseNumber(buffer.slice(0, 2));
        channelMappings = [];
        offset = 4;
        for (i = _i = 0; _i <= 1; i = ++_i) {
          leftGain = this._parseNumber(buffer.slice(offset + 1, +(offset + 3) + 1 || 9e9));
          rightGain = this._parseNumber(buffer.slice(offset + 5, +(offset + 7) + 1 || 9e9));
          this._merge(this.state.audio.master, {
            leftLevel: leftGain / 8388607,
            rightLevel: rightGain / 8388607
          });
          offset += 16;
        }
        for (i = _j = 0; 0 <= numberOfChannels ? _j < numberOfChannels : _j > numberOfChannels; i = 0 <= numberOfChannels ? ++_j : --_j) {
          channelMappings.push(buffer[offset] << 8 | buffer[offset + 1]);
          offset += 2;
        }
        _results = [];
        for (i = _k = 0; 0 <= numberOfChannels ? _k < numberOfChannels : _k > numberOfChannels; i = 0 <= numberOfChannels ? ++_k : --_k) {
          leftGain = this._parseNumber(buffer.slice(offset + 1, +(offset + 3) + 1 || 9e9));
          rightGain = this._parseNumber(buffer.slice(offset + 5, +(offset + 7) + 1 || 9e9));
          this._merge(this.state.audio.channels[channelMappings[i]], {
            leftLevel: leftGain / 8388607,
            rightLevel: rightGain / 8388607
          });
          _results.push(offset += 16);
        }
        return _results;
    }
  };

  ATEM.prototype._parseNumber = function(bytes) {
    var byte, i, num, _i, _len;
    num = 0;
    for (i = _i = 0, _len = bytes.length; _i < _len; i = ++_i) {
      byte = bytes[i];
      num += byte;
      if (i < bytes.length - 1) {
        num = num << 8;
      }
    }
    return num;
  };

  ATEM.prototype._parseString = function(bytes) {
    var char, str, _i, _len;
    str = '';
    for (_i = 0, _len = bytes.length; _i < _len; _i++) {
      char = bytes[_i];
      if (char === 0) {
        break;
      }
      str += String.fromCharCode(char);
    }
    return str;
  };

  ATEM.prototype._bufferToArray = function(buffers) {
    var arr, buffer, _i, _len;
    arr = [];
    for (_i = 0, _len = buffers.length; _i < _len; _i++) {
      buffer = buffers[_i];
      arr.push(buffer);
    }
    return arr;
  };

  ATEM.prototype._numberToBytes = function(number, numberOfBytes) {
    var bytes, i, shift, _i;
    bytes = [];
    for (i = _i = 0; 0 <= numberOfBytes ? _i < numberOfBytes : _i > numberOfBytes; i = 0 <= numberOfBytes ? ++_i : --_i) {
      shift = numberOfBytes - i - 1;
      bytes.push((number >> (8 * shift)) & 0xFF);
    }
    return bytes;
  };

  ATEM.prototype._stringToBytes = function(str) {
    var bytes, i, _i, _ref;
    bytes = [];
    for (i = _i = 0, _ref = str.length; 0 <= _ref ? _i < _ref : _i > _ref; i = 0 <= _ref ? ++_i : --_i) {
      bytes.push(str.charCodeAt(i));
    }
    return bytes;
  };

  ATEM.prototype._merge = function(obj1, obj2) {
    var key2, _results;
    if (obj2 == null) {
      obj2 = {};
    }
    _results = [];
    for (key2 in obj2) {
      if (obj2.hasOwnProperty(key2)) {
        _results.push(obj1[key2] = obj2[key2]);
      } else {
        _results.push(void 0);
      }
    }
    return _results;
  };

  ATEM.prototype.changeProgramInput = function(input) {
    return this._sendCommand('CPgI', [0x00, 0x00, input >> 8, input & 0xFF]);
  };

  ATEM.prototype.changePreviewInput = function(input) {
    return this._sendCommand('CPvI', [0x00, 0x00, input >> 8, input & 0xFF]);
  };

  ATEM.prototype.changeAuxInput = function(aux, input) {
    return this._sendCommand('CAuS', [0x01, aux, input >> 8, input & 0xFF]);
  };

  ATEM.prototype.fadeToBlack = function() {
    return this._sendCommand('FtbA', [0x00, 0x02, 0x58, 0x99]);
  };

  ATEM.prototype.autoTransition = function() {
    return this._sendCommand('DAut', [0x00, 0x00, 0x00, 0x00]);
  };

  ATEM.prototype.cutTransition = function() {
    return this._sendCommand('DCut', [0x00, 0xef, 0xbf, 0x5f]);
  };

  ATEM.prototype.changeTransitionPosition = function(position) {
    this._sendCommand('CTPs', [0x00, 0xe4, position / 256, position % 256]);
    if (position === 10000) {
      return this._sendCommand('CTPs', [0x00, 0xf6, 0x00, 0x00]);
    }
  };

  ATEM.prototype.changeTransitionPreview = function(state) {
    return this._sendCommand('CTPr', [0x00, state, 0x00, 0x00]);
  };

  ATEM.prototype.changeTransitionType = function(type) {
    return this._sendCommand('CTTp', [0x01, 0x00, type, 0x02]);
  };

  ATEM.prototype.changeDownstreamKeyOn = function(number, state) {
    return this._sendCommand('CDsL', [number, state, 0xff, 0xff]);
  };

  ATEM.prototype.changeDownstreamKeyTie = function(number, state) {
    return this._sendCommand('CDsT', [number, state, 0xff, 0xff]);
  };

  ATEM.prototype.changeUpstreamKeyState = function(number, state) {
    return this._sendCommand('CKOn', [0x00, number, state, 0x90]);
  };

  ATEM.prototype.changeUpstreamKeyNextBackground = function(state) {
    var states;
    this.state.video.upstreamKeyNextBackground = state;
    states = this.state.video.upstreamKeyNextBackground + (this.state.video.upstreamKeyNextState[0] << 1) + (this.state.video.upstreamKeyNextState[1] << 2) + (this.state.video.upstreamKeyNextState[2] << 3) + (this.state.video.upstreamKeyNextState[3] << 4);
    return this._sendCommand('CTTp', [0x02, 0x00, 0x6a, states]);
  };

  ATEM.prototype.changeUpstreamKeyNextState = function(number, state) {
    var states;
    this.state.video.upstreamKeyNextState[number] = state;
    states = this.state.video.upstreamKeyNextBackground + (this.state.video.upstreamKeyNextState[0] << 1) + (this.state.video.upstreamKeyNextState[1] << 2) + (this.state.video.upstreamKeyNextState[2] << 3) + (this.state.video.upstreamKeyNextState[3] << 4);
    return this._sendCommand('CTTp', [0x02, 0x00, 0x6a, states]);
  };

  ATEM.prototype.changeAudioMasterGain = function(gain) {
    gain = gain * AUDIO_GAIN_RATE;
    return this._sendCommand('CAMM', [0x01, 0x00, gain / 256, gain % 256, 0x00, 0x00, 0x00, 0x00]);
  };

  ATEM.prototype.changeAudioChannelGain = function(channel, gain) {
    gain = gain * AUDIO_GAIN_RATE;
    return this._sendCommand('CAMI', [0x02, 0x00, channel / 256, channel % 256, 0x00, 0x00, gain / 256, gain % 256, 0x00, 0x00, 0x00, 0x00]);
  };

  ATEM.prototype.changeAudioChannelState = function(channel, status) {
    return this._sendCommand('CAMI', [0x01, 0x00, channel >> 8, channel & 0xFF, status, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00]);
  };

  ATEM.prototype.sendAudioLevelNumber = function(enable) {
    if (enable == null) {
      enable = true;
    }
    return this._sendCommand('SALN', [enable, 0x00, 0x00, 0x00]);
  };

  ATEM.prototype.startRecordMacro = function(number, name, description) {
    var bytes, descriptionLength, nameLength;
    nameLength = (name != null ? name.length : void 0) || 0;
    descriptionLength = (description != null ? description.length : void 0) || 0;
    bytes = [0x00, number];
    bytes = bytes.concat(this._numberToBytes(nameLength, 2));
    bytes = bytes.concat(this._numberToBytes(descriptionLength, 2));
    if (nameLength > 0) {
      bytes = bytes.concat(this._stringToBytes(name));
    }
    if (descriptionLength > 0) {
      bytes = bytes.concat(this._stringToBytes(description));
    }
    return this._sendCommand('MSRc', bytes);
  };

  ATEM.prototype.stopRecordMacro = function() {
    return this._sendCommand('MAct', [0xFF, 0xFF, 0x02, 0x81]);
  };

  ATEM.prototype.runMacro = function(number) {
    return this._sendCommand('MAct', [0x00, number, 0x00, 0x7d]);
  };

  ATEM.prototype.deleteMacro = function(number) {
    return this._sendCommand('MAct', [0x00, number, 0x05, 0x00]);
  };

  return ATEM;

})();

module.exports = ATEM;
